(* This module helps report errors. *)

type location =
    Lexing.position * Lexing.position

(* A dummy location. *)

val dummy: location

(* [override inherited_loc (new_loc, 'a)] returns a pair of a new
   location, which is either the new location if it is non-trivial,
   and the inherited location otherwise, and ['a]. *)

val override: location -> location * 'a -> location * 'a

(* [error locs msg] displays the error message [msg], referring to the
   locations [locs], and stops the program. *)

val error: location list -> string -> 'a

(* [errorb] is analogous to [error], but accepts a lexing buffer. *)

val errorb: Lexing.lexbuf -> string -> 'a

(* [signal locs msg] is analogous to [error], but does not immediately stop
   the program. In order to stop the program, one invokes [signaled()], which
   stops the program if any call to [signal] was previously issued. *)

val signal: location list -> string -> unit
val signaled: unit -> unit

